sub LegoInit(PortNum)
  _LegoSimulation=False
  _FAST = 78
  _SLOW = 70
  _STOP = 0
  _RightTurnTime=900
  _LeftTurnTime=900
  _AdvanceTime=1500
  _DebugTime=120
  _TurretSpeed = 66
  _TurretTime = 900
  _LegoDebug = false
  Dim SS[3]
  SetCommPort PortNum,br115200
return

//-----------------------------------------------------
sub LegoDriveMotors(A,B)
   //code,code,port,Power,turn on,reg,t ratio,run state,run forever
   SerialOut 12,0,0x80,4,0,A,1,0,0,0x20,0,0,0,0
   SerialOut 12,0,0x80,4,1,B,1,0,0,0x20,0,0,0,0
return

//-----------------------------------------------------   
sub LegoThirdMotor(A)
   SerialOut 12,0,0x80,4,2,A,1,0,0,0x20,0,0,0,0
return

//-----------------------------------------------------
sub Wait(t)
  for i= 1 to t
    delay 1
  next
return

//-----------------------------------------------------
sub LegoLineSensor(P,&S)
   if _LegoDebug 
     call Wait(_DebugTime)
     gosub LegoHalt
   endif
   SerialOut 3,0,0,7,P-1
   repeat
     CheckSerBuffer NumBytes      
   until NumBytes=18
   serin REPLY
   S = (GetStrByte(REPLY,length(REPLY)) !=2)
   if _LegoDebug
     GetLineWidth lw \ LineWidth 1
     rectangle 0,550,799,599,black,white
     xyString 50,570,"Line Sensor (Port ",P,") = ",S
     input "Press ENTER", A
     LineWidth lw
   endif      
return

//-----------------------------------------------------
sub LegoSoundSensor(P,Sensitivity,&S)
   if _LegoDebug 
     call Wait(_DebugTime)
     gosub LegoHalt
     GetLineWidth lw \ LineWidth 1
     rectangle 0,550,799,599,black,white
     xyString 50,570,"Sound Sensor: Enter 1 ",\
                 "for sound, 0 for no sound."
     input "Type 0 or 1 then Press ENTER", S   
     LineWidth lw
   else
     Sensitivity = Limit(Sensitivity,0,150)
     // read sound sensor three times
     for T2= 0 to 2
       // request a read
       SerialOut 3,0,0,7,P-1
       repeat
         CheckSerBuffer T1 
       until T1=18
       serin d
       SS[T2]=GetStrByte(d,17)
     next
     // determine if there has been a CHANGE in sound level
     Smax=SS[0] \ Smin=SS[0]
     for T2=1 to 2
       if SS[T2]>Smax then Smax=SS[T2]
       if SS[T2]<Smin then Smin=SS[T2]
     next
     S = ((Smax-Smin) > Sensitivity)
   endif  
return

//-----------------------------------------------------
sub LegoBumperSensor(P,&S)
   if _LegoDebug 
     call Wait(_DebugTime)
     gosub LegoHalt
   endif
   SerialOut 3,0,0,7,P-1
   repeat
     CheckSerBuffer T1 
   until T1=18
   serin d
   S =  (GetStrByte(d,15) != 0)
   if _LegoDebug
     GetLineWidth lw \ LineWidth 1
     rectangle 0,550,799,599,black,white
     xyString 50,570,"Bumper Sensor = ",S
     input "Press ENTER", A
     LineWidth lw
   endif
return

//-----------------------------------------------------
sub LegoRangeSensor(P,&S)
   P--
   if _LegoDebug 
     call Wait(_DebugTime)
     gosub LegoHalt
   endif
   SerialOut 7,0,0,0xF,P,2,1,2,0x42
   repeat
      CheckSerBuffer T1 
   until T1=5
   serin d
   SerialOut 3,0,0,0x10,P
   repeat
      CheckSerBuffer T1 
   until T1=22
   serin d
   S=GetStrByte(d,7)
   S = Round(S/2.25)-5
   if _LegoDebug
     GetLineWidth lw \ LineWidth 1
     rectangle 0,550,799,599,black,white
     xyString 50,570,"Range Sensor = ", S
     input "Press ENTER", A
     LineWidth lw
   endif
return

//-----------------------------------------------------
sub LegoBumperInit(P)
   //                  port,switch,boolean
   SerialOut 5,0,0x80,5,P-1,1,0x20
   delay 100   
return

//-----------------------------------------------------
sub LegoSoundInit(P)
   //                   port,type,mode
   SerialOut 5,0,0x80,5,P-1,0x07,0
   delay 100
return

//-----------------------------------------------------
sub LegoRangeInit(P)
   //                  port,ultrasonic,raw
   SerialOut 5,0,0x80,5,P-1,0xb,0
   delay 100
   call LegoRangeSensor(P,S)
   call LegoRangeSensor(P,S)
return

//-----------------------------------------------------
sub LegoLineInit(P)
   //                   port,light active,raw
   SerialOut 5,0,0x80,5,P-1,5,0
   delay 100
return

//-----------------------------------------------------
sub Lego3LineInit()
  for i=1 to 3
    call LegoLineInit(i)
  next
return

//-----------------------------------------------------
sub Lego3LineSensors(&P)
  P=0
  if _LegoDebug 
    call Wait(_DebugTime)
    gosub LegoHalt
  endif
  for i=0 to 2      
    SerialOut 3,0,0,7,i
    repeat
      CheckSerBuffer NumBytes      
    until NumBytes=18
    serin REPLY
    if GetStrByte(REPLY,Length(REPLY)) !=2 then P+=(1<< i)
  next
  if _LegoDebug
     GetLineWidth lw \ LineWidth 1
     rectangle 0,550,799,599,black,white
     xyString 50,570,"Line Sensors (BIN) = ",Bin(P,3)
     input "Press ENTER", A
     LineWidth lw
   endif
return  

//-----------------------------------------------------
sub LegoPen(P)
return

//-----------------------------------------------------
LegoAdvance:
  call LegoDriveMotors(_SLOW,_SLOW)
  call Wait(_AdvanceTime)
  call LegoDriveMotors(_STOP,_STOP)
return

//-----------------------------------------------------
LegoRetreat:
  call LegoDriveMotors(-_SLOW,-_SLOW)
  call Wait(_AdvanceTime)
  call LegoDriveMotors(_STOP,_STOP) 
return 

//-----------------------------------------------------
LegoFaceLeft:
  call LegoDriveMotors(-_SLOW,_SLOW)
  call Wait(_LeftTurnTime)
  call LegoDriveMotors(_STOP,_STOP)
return

//-----------------------------------------------------
LegoFaceRight:
  call LegoDriveMotors(_SLOW,-_SLOW)
  call Wait(_RightTurnTime)
  call LegoDriveMotors(_STOP,_STOP)
return  

//-----------------------------------------------------
DrawLine:
return 

//-----------------------------------------------------
LegoHalt:
  call LegoDriveMotors(_STOP,_STOP)
return

//-----------------------------------------------------
LegoLookLeft:
  call LegoThirdMotor(_TurretSpeed)
  delay _TurretTime
  call LegoThirdMotor(0)
  delay 200
return

//-----------------------------------------------------
LegoLookAhead:
  call LegoThirdMotor(-_TurretSpeed)
  delay _TurretTime
  call LegoThirdMotor(0)
  delay 200
return

//-----------------------------------------------------
LegoHardRight:
  call LegoDriveMotors(_FAST,_STOP)
return

//-----------------------------------------------------
LegoEasyRight:
  call LegoDriveMotors(_FAST,_SLOW)
return

//-----------------------------------------------------
LegoHardLeft:
  call LegoDriveMotors(_STOP,_FAST)
return

//-----------------------------------------------------
LegoEasyLeft:
  call LegoDriveMotors(_SLOW,_FAST)
return
//-----------------------------------------------------
